@@ -27,14 +27,12 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
+#include <gio/gio.h>
 
 #include <libempathy/empathy-debug.h>
 
-
 #include "empathy-avatar-chooser.h"
 #include "empathy-conf.h"
-#include "empathy-preferences.h"
 #include "empathy-ui-utils.h"
 
 #define DEBUG_DOMAIN "AvatarChooser"
@@ -329,46 +327,57 @@ avatar_chooser_drag_data_received_cb (GtkWidget          *widget,
 
 	target_type = gdk_atom_name (selection_data->target);
 	if (!strcmp (target_type, URI_LIST_TYPE)) {
-		GnomeVFSHandle   *handle = NULL;
-		GnomeVFSResult    result;
-		GnomeVFSFileInfo  info;
-		gchar            *uri;
+		GFile            *file;
+		GFileInputStream *input_stream;
 		gchar            *nl;
 		gchar            *data = NULL;
 
 		nl = strstr (selection_data->data, "\r\n");
 		if (nl) {
+			gchar *uri;
+
 			uri = g_strndup (selection_data->data,
 					 nl - (gchar*) selection_data->data);
+
+			file = g_file_new_for_uri (uri);
+			g_free (uri);
 		} else {
-			uri = g_strdup (selection_data->data);
+			file = g_file_new_for_uri (selection_data->data);
 		}
 
-		result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
-		if (result == GNOME_VFS_OK) {
-			result = gnome_vfs_get_file_info_from_handle (handle,
-								      &info,
-								      GNOME_VFS_FILE_INFO_DEFAULT);
-			if (result == GNOME_VFS_OK) {
-				GnomeVFSFileSize data_size;
+		input_stream = g_file_read (file, NULL, NULL);
 
-				data = g_malloc (info.size);
+		if (input_stream != NULL) {
+			GFileInfo *info;
+			
+			info = g_file_query_info (file,
+						  G_FILE_ATTRIBUTE_STANDARD_SIZE,
+						  0, NULL, NULL);
+			if (info != NULL) {
+				goffset size;
+				gssize bytes_read;
+				
+				size = g_file_info_get_size (info);
+				data = g_malloc (size);
 
-				result = gnome_vfs_read (handle, data, info.size, &data_size);
-				if (result == GNOME_VFS_OK) {
+				bytes_read = g_input_stream_read (G_INPUT_STREAM (input_stream),
+								  data, size,
+								  NULL, NULL);
+				if (bytes_read != -1) {
 					avatar_chooser_set_image_from_data (chooser,
 									    data,
-									    data_size);
+									    (gsize) bytes_read);
 					handled = TRUE;
-				} else {
-					g_free (data);
 				}
+
+				g_free (data);
+				g_object_unref (info);
 			}
 
-			gnome_vfs_close (handle);
+			g_object_unref (input_stream);
 		}
-
-		g_free (uri);
+		
+		g_object_unref (file);
 	}
 
 	gtk_drag_finish (context, handled, FALSE, time);
