@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2005-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,6 +30,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#include <telepathy-glib/util.h>
 #include <libempathy/empathy-debug.h>
 
 #include "empathy-contact-list-store.h"
@@ -90,10 +91,6 @@ static void             contact_list_store_set_property              (GObject
 								      guint                          param_id,
 								      const GValue                  *value,
 								      GParamSpec                    *pspec);
-static gboolean         contact_list_store_finalize_foreach          (GtkTreeModel                  *model,
-								      GtkTreePath                   *path,
-								      GtkTreeIter                   *iter,
-								      gpointer                       user_data);
 static void             contact_list_store_setup                     (EmpathyContactListStore       *store);
 static gboolean         contact_list_store_inibit_active_cb          (EmpathyContactListStore       *store);
 static void             contact_list_store_members_changed_cb        (EmpathyContactList            *list_iface,
@@ -209,8 +206,7 @@ contact_list_store_set_contact_list (EmpathyContactListStore *store,
 	priv->list = g_object_ref (list_iface);
 
 	/* Let a chance to have all properties set before populating */
-	g_idle_add (contact_list_store_iface_setup,
-		    store);
+	g_idle_add (contact_list_store_iface_setup, store);
 }
 
 static void
@@ -292,23 +288,26 @@ empathy_contact_list_store_init (EmpathyContactListStore *store)
 static void
 contact_list_store_finalize (GObject *object)
 {
-	EmpathyContactListStorePriv *priv;
+	EmpathyContactListStorePriv *priv = GET_PRIV (object);
+	GList                       *contacts, *l;
 
-	priv = GET_PRIV (object);
-
-	gtk_tree_model_foreach (GTK_TREE_MODEL (object),
-				(GtkTreeModelForeachFunc) contact_list_store_finalize_foreach,
-				object);
-
-	if (priv->list) {
-		g_signal_handlers_disconnect_by_func (priv->list,
-						      G_CALLBACK (contact_list_store_members_changed_cb),
+	contacts = empathy_contact_list_get_members (priv->list);
+	for (l = contacts; l; l = l->next) {
+		g_signal_handlers_disconnect_by_func (l->data,
+						      G_CALLBACK (contact_list_store_contact_updated_cb),
 						      object);
-		g_signal_handlers_disconnect_by_func (priv->list,
-						      G_CALLBACK (contact_list_store_groups_changed_cb),
-						      object);
-		g_object_unref (priv->list);
+
+		g_object_unref (l->data);
 	}
+	g_list_free (contacts);
+
+	g_signal_handlers_disconnect_by_func (priv->list,
+					      G_CALLBACK (contact_list_store_members_changed_cb),
+					      object);
+	g_signal_handlers_disconnect_by_func (priv->list,
+					      G_CALLBACK (contact_list_store_groups_changed_cb),
+					      object);
+	g_object_unref (priv->list);
 
 	if (priv->inhibit_active) {
 		g_source_remove (priv->inhibit_active);
@@ -725,28 +724,6 @@ empathy_contact_list_store_search_equal_func (GtkTreeModel *model,
 	return ret;
 }
 
-static gboolean
-contact_list_store_finalize_foreach (GtkTreeModel *model,
-				     GtkTreePath  *path,
-				     GtkTreeIter  *iter,
-				     gpointer      user_data)
-{
-	EmpathyContactListStore *store = user_data;
-	EmpathyContact          *contact = NULL;
-
-	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
-			    -1);
-
-	if (contact) {
-		g_signal_handlers_disconnect_by_func (contact,
-						      G_CALLBACK (contact_list_store_contact_updated_cb),
-						      store);
-	}
-
-	return FALSE;
-}
-
 static void
 contact_list_store_setup (EmpathyContactListStore *store)
 {
@@ -880,7 +857,8 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 
 	priv = GET_PRIV (store);
 	
-	if (!priv->show_offline && !empathy_contact_is_online (contact)) {
+	if (G_STR_EMPTY (empathy_contact_get_name (contact)) ||
+	    (!priv->show_offline && !empathy_contact_is_online (contact))) {
 		return;
 	}
 
@@ -976,6 +954,7 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	gboolean                    do_remove = FALSE;
 	gboolean                    do_set_active = FALSE;
 	gboolean                    do_set_refresh = FALSE;
+	gboolean                    show_avatar = FALSE;
 	GdkPixbuf                  *pixbuf_avatar;
 
 	priv = GET_PRIV (store);
@@ -1070,12 +1049,15 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 		set_model = TRUE;
 	}
 
+	if (priv->show_avatars && !priv->is_compact) {
+		show_avatar = TRUE;
+	}
 	pixbuf_avatar = empathy_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
 	for (l = iters; l && set_model; l = l->next) {
 		gtk_tree_store_set (GTK_TREE_STORE (store), l->data,
 				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, empathy_icon_name_for_contact (contact),
 				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR, pixbuf_avatar,
-				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, priv->show_avatars,
+				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
 				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS, empathy_contact_get_status (contact),
 				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
@@ -1236,7 +1218,7 @@ contact_list_store_get_group_foreach (GtkTreeModel *model,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
 			    -1);
 
-	if (is_group && strcmp (str, fg->name) == 0) {
+	if (is_group && !tp_strdiff (str, fg->name)) {
 		fg->found = TRUE;
 		fg->iter = *iter;
 	}
@@ -1484,15 +1466,14 @@ contact_list_store_find_contact_foreach (GtkTreeModel *model,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
 			    -1);
 
-	if (!contact) {
-		return FALSE;
-	}
-
-	if (empathy_contact_equal (contact, fc->contact)) {
+	if (contact == fc->contact) {
 		fc->found = TRUE;
 		fc->iters = g_list_append (fc->iters, gtk_tree_iter_copy (iter));
 	}
-	g_object_unref (contact);
+
+	if (contact) {
+		g_object_unref (contact);
+	}
 
 	return FALSE;
 }
