@@ -8,17 +8,17 @@
 
 G_BEGIN_DECLS
 
-#include <libempathy-gtk/empathy-contact-widget.h>
-#define EMPATHY_TYPE_CONTACT_WIDGET_FLAGS empathy_contact_widget_flags_get_type()
-GType empathy_contact_widget_flags_get_type (void);
 #include <libempathy-gtk/empathy-contact-list-store.h>
 #define EMPATHY_TYPE_CONTACT_LIST_STORE_SORT empathy_contact_list_store_sort_get_type()
 GType empathy_contact_list_store_sort_get_type (void);
 #define EMPATHY_TYPE_CONTACT_LIST_STORE_COL empathy_contact_list_store_col_get_type()
 GType empathy_contact_list_store_col_get_type (void);
-#include <libempathy-gtk/empathy-chat-view.h>
-#define EMPATHY_TYPE_TYPE block_type_get_type()
-GType block_type_get_type (void);
+#include <libempathy-gtk/empathy-contact-list-view.h>
+#define EMPATHY_TYPE_CONTACT_LIST_FEATURES empathy_contact_list_features_get_type()
+GType empathy_contact_list_features_get_type (void);
+#include <libempathy-gtk/empathy-contact-widget.h>
+#define EMPATHY_TYPE_CONTACT_WIDGET_FLAGS empathy_contact_widget_flags_get_type()
+GType empathy_contact_widget_flags_get_type (void);
 G_END_DECLS
 
 #endif /* __LIBEMPATHY_GTK_ENUM_TYPES_H__ */
