@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -93,9 +93,6 @@ static void     log_window_chats_populate                  (EmpathyLogWindow *wi
 static void     log_window_chats_setup                     (EmpathyLogWindow *window);
 static void     log_window_chats_accounts_changed_cb       (GtkWidget        *combobox,
 							    EmpathyLogWindow *window);
-static void     log_window_chats_new_message_cb            (EmpathyContact   *own_contact,
-							    EmpathyMessage   *message,
-							    EmpathyLogWindow *window);
 static void     log_window_chats_set_selected              (EmpathyLogWindow *window,
 							    McAccount        *account,
 							    const gchar      *chat_id,
@@ -147,6 +144,7 @@ empathy_log_window_show (McAccount   *account,
 	GList                  *accounts;
 	gint                    account_num;
 	GladeXML               *glade;
+	gchar                  *filename;
 
 	if (window) {
 		gtk_window_present (GTK_WINDOW (window->window));
@@ -163,7 +161,9 @@ empathy_log_window_show (McAccount   *account,
 	window = g_new0 (EmpathyLogWindow, 1);
 	window->log_manager = empathy_log_manager_new ();
 
-	glade = empathy_glade_get_file ("empathy-log-window.glade",
+	filename = empathy_file_lookup ("empathy-log-window.glade",
+					"libempathy-gtk");
+	glade = empathy_glade_get_file (filename,
 				       "log_window",
 				       NULL,
 				       "log_window", &window->window,
@@ -180,6 +180,8 @@ empathy_log_window_show (McAccount   *account,
 				       "treeview_chats", &window->treeview_chats,
 				       "scrolledwindow_chats", &window->scrolledwindow_chats,
 				       NULL);
+	g_free (filename);
+
 	empathy_glade_connect (glade,
 			      window,
 			      "log_window", "destroy", log_window_destroy_cb,
@@ -269,10 +271,6 @@ static void
 log_window_destroy_cb (GtkWidget       *widget,
 		       EmpathyLogWindow *window)
 {
-	g_signal_handlers_disconnect_by_func (window->log_manager,
-					      log_window_chats_new_message_cb,
-					      window);
-
 	g_free (window->last_find);
 	g_object_unref (window->log_manager);
 
@@ -737,17 +735,6 @@ log_window_chats_accounts_changed_cb (GtkWidget       *combobox,
 	log_window_chats_populate (window);
 }
 
-static void
-log_window_chats_new_message_cb (EmpathyContact   *own_contact,
-				 EmpathyMessage   *message,
-				 EmpathyLogWindow *window)
-{
-	empathy_chat_view_append_message (window->chatview_chats, message);
-
-	/* Scroll to the most recent messages */
-	empathy_chat_view_scroll_down (window->chatview_chats);
-}
-
 static void
 log_window_chats_set_selected  (EmpathyLogWindow *window,
 				McAccount       *account,
@@ -972,9 +959,6 @@ log_window_chats_get_messages (EmpathyLogWindow *window,
 	/* Turn back on scrolling */
 	empathy_chat_view_scroll (window->chatview_find, TRUE);
 
-	/* Scroll to the most recent messages */
-	empathy_chat_view_scroll_down (window->chatview_chats);
-
 	/* Give the search entry main focus */
 	gtk_widget_grab_focus (window->entry_chats);
 
