@@ -37,12 +37,11 @@
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-debug.h>
-#include <libempathy/empathy-marshal.h>
+#include <libempathy/empathy-status-presets.h>
 
 #include "empathy-ui-utils.h"
 #include "empathy-images.h"
 #include "empathy-presence-chooser.h"
-#include "empathy-status-presets.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_PRESENCE_CHOOSER, EmpathyPresenceChooserPriv))
 
@@ -381,7 +380,7 @@ presence_chooser_scroll_event_cb (EmpathyPresenceChooser *chooser,
 		/* If we didn't get any match at all, it means the last state
 		 * was a custom one. Just switch to the first one.
 		 */
-		status = empathy_presence_state_get_default_status (states[0]);
+		status = empathy_presence_get_default_message (states[0]);
 
 		presence_chooser_reset_scroll_timeout (chooser);
 		empathy_idle_set_presence (priv->idle, states[0], status);
@@ -404,7 +403,7 @@ presence_chooser_get_presets (EmpathyPresenceChooser *chooser)
 		StateAndStatus *sas;
 		const gchar    *status;
 
-		status = empathy_presence_state_get_default_status (states[i]);
+		status = empathy_presence_get_default_message (states[i]);
 		sas = presence_chooser_state_and_status_new (states[i], status);
 		list = g_list_prepend (list, sas);
 
@@ -455,7 +454,7 @@ presence_chooser_flash_timeout_cb (EmpathyPresenceChooser *chooser)
 	}
 
 	gtk_image_set_from_icon_name (GTK_IMAGE (priv->image),
-				      empathy_icon_name_for_presence_state (state),
+				      empathy_icon_name_for_presence (state),
 				      GTK_ICON_SIZE_MENU);
 
 	on = !on;
@@ -500,7 +499,7 @@ presence_chooser_flash_stop (EmpathyPresenceChooser *chooser,
 	}
 
 	gtk_image_set_from_icon_name (GTK_IMAGE (priv->image),
-				      empathy_icon_name_for_presence_state (state),
+				      empathy_icon_name_for_presence (state),
 				      GTK_ICON_SIZE_MENU);
 
 	priv->last_state = state;
@@ -665,7 +664,7 @@ empathy_presence_chooser_create_menu (void)
 	for (i = 0; i < G_N_ELEMENTS (states); i += 2) {
 		GList       *list, *l;
 
-		status = empathy_presence_state_get_default_status (states[i]);
+		status = empathy_presence_get_default_message (states[i]);
 		presence_chooser_menu_add_item (menu,
 						status,
 						states[i]);
@@ -714,7 +713,7 @@ presence_chooser_menu_add_item (GtkWidget   *menu,
 	const gchar *icon_name;
 
 	item = gtk_image_menu_item_new_with_label (str);
-	icon_name = empathy_icon_name_for_presence_state (state);
+	icon_name = empathy_icon_name_for_presence (state);
 
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (presence_chooser_noncustom_activate_cb),
@@ -808,6 +807,8 @@ presence_chooser_dialog_status_changed_cb (GtkWidget           *widget,
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (dialog->comboboxentry_message),
 				 GTK_TREE_MODEL (store));
+
+	g_object_unref (store);
 }
 
 static void
@@ -889,8 +890,8 @@ presence_chooser_dialog_setup (CustomMessageDialog *dialog)
 
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
-				    COL_ICON, empathy_icon_name_for_presence_state (states[i]),
-				    COL_LABEL, empathy_presence_state_get_default_status (states[i]),
+				    COL_ICON, empathy_icon_name_for_presence (states[i]),
+				    COL_LABEL, empathy_presence_get_default_message (states[i]),
 				    COL_PRESENCE, states[i],
 				    -1);
 	}
@@ -898,17 +899,28 @@ presence_chooser_dialog_setup (CustomMessageDialog *dialog)
 	gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->combobox_status), 0);
 }
 
+static void
+presence_chooser_dialog_response_cb (GtkWidget           *widget,
+				     gint                 response,
+				     CustomMessageDialog *dialog)
+{
+	if (response == GTK_RESPONSE_APPLY) {
+		McPresence   state;
+		const gchar *text;
+
+		state = presence_chooser_dialog_get_selected (dialog);
+		text = gtk_entry_get_text (GTK_ENTRY (dialog->entry_message));
+
+		presence_chooser_set_state (state, text);
+	}
+
+	gtk_widget_destroy (widget);
+}
+
 static void
 presence_chooser_dialog_destroy_cb (GtkWidget           *widget,
 				    CustomMessageDialog *dialog)
 {
-	McPresence   state;
-	const gchar *text;
-
-	state = presence_chooser_dialog_get_selected (dialog);
-	text = gtk_entry_get_text (GTK_ENTRY (dialog->entry_message));
-
-	presence_chooser_set_state (state, text);
 
 	g_free (dialog);
 	message_dialog = NULL;
@@ -918,6 +930,7 @@ static void
 presence_chooser_dialog_show (void)
 {
 	GladeXML *glade;
+	gchar    *filename;
 
 	if (message_dialog) {
 		gtk_window_present (GTK_WINDOW (message_dialog->dialog));
@@ -925,7 +938,10 @@ presence_chooser_dialog_show (void)
 	}
 
 	message_dialog = g_new0 (CustomMessageDialog, 1);
-	glade = empathy_glade_get_file ("empathy-presence-chooser.glade",
+
+	filename = empathy_file_lookup ("empathy-presence-chooser.glade",
+					"libempathy-gtk");
+	glade = empathy_glade_get_file (filename,
 				       "custom_message_dialog",
 				       NULL,
 				       "custom_message_dialog", &message_dialog->dialog,
@@ -933,10 +949,12 @@ presence_chooser_dialog_show (void)
 				       "comboboxentry_message", &message_dialog->comboboxentry_message,
 				       "combobox_status", &message_dialog->combobox_status,
 				       NULL);
+	g_free (filename);
+
 	empathy_glade_connect (glade,
 			       message_dialog,
 			       "custom_message_dialog", "destroy", presence_chooser_dialog_destroy_cb,
-			       "custom_message_dialog", "response", gtk_widget_destroy,
+			       "custom_message_dialog", "response", presence_chooser_dialog_response_cb,
 			       "combobox_status", "changed", presence_chooser_dialog_status_changed_cb,
 			       "checkbutton_save", "toggled", presence_chooser_dialog_save_toggled_cb,
 			       NULL);
