@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2002-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,8 +35,8 @@
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include <libmissioncontrol/mc-profile.h>
 
@@ -60,24 +60,16 @@ get_glade_file (const gchar *filename,
 		const gchar *first_required_widget,
 		va_list      args)
 {
-	gchar      *path;
 	GladeXML   *gui;
 	const char *name;
 	GtkWidget **widget_ptr;
 
-	path = g_build_filename (UNINSTALLED_GLADE_DIR, filename, NULL);
-	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
-		g_free (path);
-		path = g_build_filename (DATADIR, "empathy", filename, NULL);
-	}
-	empathy_debug (DEBUG_DOMAIN, "Loading glade file %s", path);
+	empathy_debug (DEBUG_DOMAIN, "Loading glade file %s", filename);
 
-	gui = glade_xml_new (path, root, domain);
-	g_free (path);
+	gui = glade_xml_new (filename, root, domain);
 
 	if (!gui) {
 		g_warning ("Couldn't find necessary glade file '%s'", filename);
-		return NULL;
 	}
 
 	for (name = first_required_widget; name; name = va_arg (args, char *)) {
@@ -114,11 +106,9 @@ empathy_glade_get_file_simple (const gchar *filename,
 
 	va_end (args);
 
-	if (!gui) {
-		return;
+	if (gui) {
+		g_object_unref (gui);
 	}
-
-	g_object_unref (gui);
 }
 
 GladeXML *
@@ -1276,6 +1266,9 @@ empathy_window_present (GtkWindow *window,
 	timestamp = gtk_get_current_event_time ();
 	gtk_window_set_skip_taskbar_hint (window, FALSE);
 	gtk_window_present_with_time (window, timestamp);
+	/* FIXME: This shouldn't be required as gtk_window_present's doc says
+	 *        it deiconify automatically. */
+	gtk_window_deiconify (window);
 }
 
 GtkWindow *
@@ -1302,31 +1295,45 @@ empathy_get_toplevel_window (GtkWidget *widget)
 static gchar *
 fixup_url (const gchar *url)
 {
-	gchar *real_url;
-
-	if (!g_str_has_prefix (url, "http://") &&
+	if (!g_str_has_prefix (url, "ghelp:") &&
 	    !strstr (url, ":/") &&
 	    !strstr (url, "@")) {
-		real_url = g_strdup_printf ("http://%s", url);
+		return g_strdup_printf ("http://%s", url);
 	} else {
-		real_url = g_strdup (url);
+		return NULL;
 	}
-
-	return real_url;
 }
 
 void
 empathy_url_show (const char *url)
 {
-	gchar          *real_url;
-	GnomeVFSResult  res;
+	gchar  *real_url;
+	GError *error = NULL;
 
 	real_url = fixup_url (url);
-	res = gnome_vfs_url_show (real_url);
-	if (res != GNOME_VFS_OK) {
-		empathy_debug (DEBUG_DOMAIN, "Couldn't show URL %s: %s",
-			      real_url,
-			      gnome_vfs_result_to_string (res));
+	if (real_url) {
+		url = real_url;
+	}
+
+	/* FIXME: this does not work for multihead, we should use
+	 * GdkAppLaunchContext or gtk_show_url, see bug #514396.
+	 */
+	g_app_info_launch_default_for_uri (url, NULL, &error);
+	if (error) {
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (NULL, 0,
+						 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+						 _("Unable to open uri"));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  error->message);
+
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_window_present (GTK_WINDOW (dialog));
+
+		g_clear_error (&error);
 	}
 
 	g_free (real_url);
