@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2004 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,8 @@
 
 #include <glib/gi18n.h>
 
+#include <telepathy-glib/util.h>
+
 #include "empathy-contact.h"
 #include "empathy-utils.h"
 #include "empathy-debug.h"
@@ -45,10 +47,13 @@ struct _EmpathyContactPriv {
 	gchar              *name;
 	EmpathyAvatar      *avatar;
 	McAccount          *account;
-	EmpathyPresence    *presence;
+	McPresence          presence;
+	gchar              *presence_message;
 	guint               handle;
 	EmpathyCapabilities capabilities;
 	gboolean            is_user;
+	guint               hash;
+	EmpathyContactReady ready;
 };
 
 static void empathy_contact_class_init (EmpathyContactClass *class);
@@ -72,11 +77,11 @@ enum {
 	PROP_AVATAR,
 	PROP_ACCOUNT,
 	PROP_PRESENCE,
-	PROP_GROUPS,
-	PROP_SUBSCRIPTION,
+	PROP_PRESENCE_MESSAGE,
 	PROP_HANDLE,
 	PROP_CAPABILITIES,
-	PROP_IS_USER
+	PROP_IS_USER,
+	PROP_READY
 };
 
 static void
@@ -124,12 +129,21 @@ empathy_contact_class_init (EmpathyContactClass *class)
 
 	g_object_class_install_property (object_class,
 					 PROP_PRESENCE,
-					 g_param_spec_object ("presence",
-							      "Contact presence",
-							      "Presence of contact",
-							      EMPATHY_TYPE_PRESENCE,
-							      G_PARAM_READWRITE));
+					 g_param_spec_uint ("presence",
+							    "Contact presence",
+							    "Presence of contact",
+							    MC_PRESENCE_UNSET,
+							    LAST_MC_PRESENCE,
+							    MC_PRESENCE_UNSET,
+							    G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_PRESENCE_MESSAGE,
+					 g_param_spec_string ("presence-message",
+							      "Contact presence message",
+							      "Presence message of contact",
+							      NULL,
+							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_HANDLE,
 					 g_param_spec_uint ("handle",
@@ -146,8 +160,8 @@ empathy_contact_class_init (EmpathyContactClass *class)
 							     "Contact Capabilities",
 							     "Capabilities of the contact",
 							     EMPATHY_TYPE_CAPABILITIES,
-							     0,
-							     G_PARAM_READWRITE));
+							     EMPATHY_CAPABILITIES_UNKNOWN,
+							     G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_IS_USER,
@@ -157,6 +171,15 @@ empathy_contact_class_init (EmpathyContactClass *class)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_READY,
+					 g_param_spec_flags ("ready",
+							     "Contact ready flags",
+							     "Flags for ready properties",
+							     EMPATHY_TYPE_CONTACT_READY,
+							     EMPATHY_CONTACT_READY_NONE,
+							     G_PARAM_READABLE));
+
 	g_type_class_add_private (object_class, sizeof (EmpathyContactPriv));
 }
 
@@ -176,15 +199,12 @@ contact_finalize (GObject *object)
 
 	g_free (priv->name);
 	g_free (priv->id);
+	g_free (priv->presence_message);
 
 	if (priv->avatar) {
 		empathy_avatar_unref (priv->avatar);
 	}
 
-	if (priv->presence) {
-		g_object_unref (priv->presence);
-	}
-
 	if (priv->account) {
 		g_object_unref (priv->account);
 	}
@@ -204,8 +224,7 @@ contact_get_property (GObject    *object,
 
 	switch (param_id) {
 	case PROP_ID:
-		g_value_set_string (value,
-				    empathy_contact_get_id (EMPATHY_CONTACT (object)));
+		g_value_set_string (value, priv->id);
 		break;
 	case PROP_NAME:
 		g_value_set_string (value,
@@ -218,7 +237,10 @@ contact_get_property (GObject    *object,
 		g_value_set_object (value, priv->account);
 		break;
 	case PROP_PRESENCE:
-		g_value_set_object (value, priv->presence);
+		g_value_set_uint (value, priv->presence);
+		break;
+	case PROP_PRESENCE_MESSAGE:
+		g_value_set_string (value, priv->presence_message);
 		break;
 	case PROP_HANDLE:
 		g_value_set_uint (value, priv->handle);
@@ -229,6 +251,9 @@ contact_get_property (GObject    *object,
 	case PROP_IS_USER:
 		g_value_set_boolean (value, priv->is_user);
 		break;
+	case PROP_READY:
+		g_value_set_flags (value, priv->ready);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -264,7 +289,11 @@ contact_set_property (GObject      *object,
 		break;
 	case PROP_PRESENCE:
 		empathy_contact_set_presence (EMPATHY_CONTACT (object),
-					     EMPATHY_PRESENCE (g_value_get_object (value)));
+					      g_value_get_uint (value));
+		break;
+	case PROP_PRESENCE_MESSAGE:
+		empathy_contact_set_presence_message (EMPATHY_CONTACT (object),
+						      g_value_get_string (value));
 		break;
 	case PROP_HANDLE:
 		empathy_contact_set_handle (EMPATHY_CONTACT (object),
@@ -284,6 +313,18 @@ contact_set_property (GObject      *object,
 	};
 }
 
+static void
+contact_set_ready_flag (EmpathyContact      *contact,
+			EmpathyContactReady  flag)
+{
+	EmpathyContactPriv *priv = GET_PRIV (contact);
+
+	if (!(priv->ready & flag)) {
+		priv->ready |= flag;
+		g_object_notify (G_OBJECT (contact), "ready");
+	}
+}
+
 EmpathyContact *
 empathy_contact_new (McAccount *account)
 {
@@ -309,15 +350,11 @@ empathy_contact_get_id (EmpathyContact *contact)
 {
 	EmpathyContactPriv *priv;
 
-	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), "");
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
 	priv = GET_PRIV (contact);
 
-	if (priv->id) {
-		return priv->id;
-	}
-
-	return "";
+	return priv->id;
 }
 
 void
@@ -331,14 +368,21 @@ empathy_contact_set_id (EmpathyContact *contact,
 
 	priv = GET_PRIV (contact);
 
-	if (priv->id && strcmp (id, priv->id) == 0) {
-		return;
+	/* We temporally ref the contact because it could be destroyed
+	 * during the signal emition */
+	g_object_ref (contact);
+	contact_set_ready_flag (contact, EMPATHY_CONTACT_READY_ID);
+	if (tp_strdiff (id, priv->id)) {
+		g_free (priv->id);
+		priv->id = g_strdup (id);
+
+		g_object_notify (G_OBJECT (contact), "id");
+		if (G_STR_EMPTY (priv->name)) {
+			g_object_notify (G_OBJECT (contact), "name");
+		}
 	}
 
-	g_free (priv->id);
-	priv->id = g_strdup (id);
-
-	g_object_notify (G_OBJECT (contact), "id");
+	g_object_unref (contact);
 }
 
 const gchar *
@@ -346,7 +390,7 @@ empathy_contact_get_name (EmpathyContact *contact)
 {
 	EmpathyContactPriv *priv;
 
-	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), "");
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
 	priv = GET_PRIV (contact);
 
@@ -364,18 +408,17 @@ empathy_contact_set_name (EmpathyContact *contact,
 	EmpathyContactPriv *priv;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
-	g_return_if_fail (name != NULL);
 
 	priv = GET_PRIV (contact);
 
-	if (priv->name && strcmp (name, priv->name) == 0) {
-		return;
+	g_object_ref (contact);
+	contact_set_ready_flag (contact, EMPATHY_CONTACT_READY_NAME);
+	if (tp_strdiff (name, priv->name)) {
+		g_free (priv->name);
+		priv->name = g_strdup (name);
+		g_object_notify (G_OBJECT (contact), "name");
 	}
-
-	g_free (priv->name);
-	priv->name = g_strdup (name);
-
-	g_object_notify (G_OBJECT (contact), "name");
+	g_object_unref (contact);
 }
 
 EmpathyAvatar *
@@ -451,12 +494,12 @@ empathy_contact_set_account (EmpathyContact *contact,
 	g_object_notify (G_OBJECT (contact), "account");
 }
 
-EmpathyPresence *
+McPresence
 empathy_contact_get_presence (EmpathyContact *contact)
 {
 	EmpathyContactPriv *priv;
 
-	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), MC_PRESENCE_UNSET);
 
 	priv = GET_PRIV (contact);
 
@@ -464,8 +507,8 @@ empathy_contact_get_presence (EmpathyContact *contact)
 }
 
 void
-empathy_contact_set_presence (EmpathyContact  *contact,
-			      EmpathyPresence *presence)
+empathy_contact_set_presence (EmpathyContact *contact,
+			      McPresence      presence)
 {
 	EmpathyContactPriv *priv;
 
@@ -477,18 +520,41 @@ empathy_contact_set_presence (EmpathyContact  *contact,
 		return;
 	}
 
-	if (priv->presence) {
-		g_object_unref (priv->presence);
-		priv->presence = NULL;
-	}
-
-	if (presence) {
-		priv->presence = g_object_ref (presence);
-	}
+	priv->presence = presence;
 
 	g_object_notify (G_OBJECT (contact), "presence");
 }
 
+const gchar *
+empathy_contact_get_presence_message (EmpathyContact *contact)
+{
+	EmpathyContactPriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+	priv = GET_PRIV (contact);
+
+	return priv->presence_message;
+}
+
+void
+empathy_contact_set_presence_message (EmpathyContact *contact,
+				      const gchar    *message)
+{
+	EmpathyContactPriv *priv = GET_PRIV (contact);
+
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+	if (!tp_strdiff (message, priv->presence_message)) {
+		return;
+	}
+
+	g_free (priv->presence_message);
+	priv->presence_message = g_strdup (message);
+
+	g_object_notify (G_OBJECT (contact), "presence-message");
+}
+
 guint
 empathy_contact_get_handle (EmpathyContact *contact)
 {
@@ -503,7 +569,7 @@ empathy_contact_get_handle (EmpathyContact *contact)
 
 void
 empathy_contact_set_handle (EmpathyContact *contact,
-			   guint          handle)
+			    guint           handle)
 {
 	EmpathyContactPriv *priv;
 
@@ -511,13 +577,13 @@ empathy_contact_set_handle (EmpathyContact *contact,
 
 	priv = GET_PRIV (contact);
 
-	if (priv->handle == handle) {
-		return;
+	g_object_ref (contact);
+	contact_set_ready_flag (contact, EMPATHY_CONTACT_READY_HANDLE);
+	if (handle != priv->handle) {
+		priv->handle = handle;
+		g_object_notify (G_OBJECT (contact), "handle");
 	}
-
-	priv->handle = handle;
-
-	g_object_notify (G_OBJECT (contact), "handle");
+	g_object_unref (contact);
 }
 
 EmpathyCapabilities
@@ -591,11 +657,7 @@ empathy_contact_is_online (EmpathyContact *contact)
 
 	priv = GET_PRIV (contact);
 
-	if (!priv->presence) {
-		return FALSE;
-	}
-
-	return (empathy_presence_get_state (priv->presence) > MC_PRESENCE_OFFLINE);
+	return (priv->presence > MC_PRESENCE_OFFLINE);
 }
 
 const gchar *
@@ -607,21 +669,11 @@ empathy_contact_get_status (EmpathyContact *contact)
 
 	priv = GET_PRIV (contact);
 
-	if (priv->presence) {
-		const gchar *status;
-
-		status = empathy_presence_get_status (priv->presence);
-		if (!status) {
-			McPresence state;
-
-			state = empathy_presence_get_state (priv->presence);
-			status = empathy_presence_state_get_default_status (state);
-		}
-
-		return status;
+	if (priv->presence_message) {
+		return priv->presence_message;
 	}
 
-	return empathy_presence_state_get_default_status (MC_PRESENCE_OFFLINE);
+	return empathy_presence_get_default_message (priv->presence);
 }
 
 gboolean
@@ -637,6 +689,18 @@ empathy_contact_can_voip (EmpathyContact *contact)
 				     EMPATHY_CAPABILITIES_VIDEO);
 }
 
+EmpathyContactReady
+empathy_contact_get_ready (EmpathyContact *contact)
+{
+	EmpathyContactPriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), FALSE);
+
+	priv = GET_PRIV (contact);
+
+	return priv->ready;
+}
+
 gboolean
 empathy_contact_equal (gconstpointer v1,
 		      gconstpointer v2)
@@ -662,15 +726,37 @@ guint
 empathy_contact_hash (gconstpointer key)
 {
 	EmpathyContactPriv *priv;
-	guint              hash;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (key), +1);
 
 	priv = GET_PRIV (EMPATHY_CONTACT (key));
 
-	hash = empathy_account_hash (empathy_contact_get_account (EMPATHY_CONTACT (key)));
-	hash += g_str_hash (empathy_contact_get_id (EMPATHY_CONTACT (key)));
+	if (priv->hash == 0) {
+		priv->hash = empathy_account_hash (priv->account) + g_str_hash (priv->id);
+	}
 
-	return hash;
+	return priv->hash;
+}
+
+static gboolean
+contact_is_ready_func (GObject  *contact,
+		       gpointer  user_data)
+{
+	EmpathyContactPriv *priv = GET_PRIV (contact);
+	EmpathyContactReady ready;
+
+	ready = GPOINTER_TO_UINT (user_data);
+
+	return (priv->ready & ready) == ready;
+}
+
+void
+empathy_contact_run_until_ready (EmpathyContact      *contact,
+				 EmpathyContactReady  ready,
+				 GMainLoop          **loop)
+{
+	empathy_run_until_ready_full (contact, "notify::ready",
+				      contact_is_ready_func, GUINT_TO_POINTER (ready),
+				      loop);
 }
 
