@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2003-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -362,7 +362,7 @@ empathy_log_manager_get_messages_for_file (EmpathyLogManager *manager,
 		is_user_str = xmlGetProp (node, "isuser");
 		msg_type_str = xmlGetProp (node, "type");
 
-		if (sender_avatar_token) {
+		if (!G_STR_EMPTY (sender_avatar_token)) {
 			avatar = empathy_avatar_new_from_cache (sender_avatar_token);
 		}
 		if (is_user_str) {
