@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,10 +24,8 @@
 
 #include <glib.h>
 
-#include <libtelepathy/tp-chan.h>
-#include <libtelepathy/tp-constants.h>
-
-#include <libmissioncontrol/mc-account.h>
+#include <telepathy-glib/channel.h>
+#include <telepathy-glib/enums.h>
 
 #include "empathy-message.h"
 #include "empathy-contact.h"
@@ -55,19 +53,23 @@ struct _EmpathyTpChatClass {
 };
 
 GType          empathy_tp_chat_get_type             (void) G_GNUC_CONST;
-EmpathyTpChat *empathy_tp_chat_new                  (McAccount          *account,
-						     TpChan             *tp_chan);
-gboolean       empathy_tp_chat_get_acknowledge      (EmpathyTpChat      *chat);
+EmpathyTpChat *empathy_tp_chat_new                  (TpChannel          *channel,
+						     gboolean            acknowledge);
+const gchar *  empathy_tp_chat_get_id               (EmpathyTpChat      *chat);
+EmpathyContact*empathy_tp_chat_get_remote_contact   (EmpathyTpChat      *chat);
+McAccount *    empathy_tp_chat_get_account          (EmpathyTpChat      *chat);
+gboolean       empathy_tp_chat_is_ready             (EmpathyTpChat      *chat);
+guint          empathy_tp_chat_get_members_count    (EmpathyTpChat      *chat);
 void           empathy_tp_chat_set_acknowledge      (EmpathyTpChat      *chat,
 						     gboolean            acknowledge);
-McAccount *    empathy_tp_chat_get_account          (EmpathyTpChat      *chat);
-TpChan *       empathy_tp_chat_get_channel          (EmpathyTpChat      *chat);
-GList *        empathy_tp_chat_get_pendings         (EmpathyTpChat      *chat);
+void           empathy_tp_chat_emit_pendings        (EmpathyTpChat      *chat);
 void           empathy_tp_chat_send                 (EmpathyTpChat      *chat,
 						     EmpathyMessage     *message);
 void           empathy_tp_chat_set_state            (EmpathyTpChat      *chat,
 						     TpChannelChatState  state);
-const gchar *  empathy_tp_chat_get_id               (EmpathyTpChat      *chat);
+void           empathy_tp_chat_set_property         (EmpathyTpChat      *chat,
+						     const gchar        *name,
+						     const GValue       *value);
 
 G_END_DECLS
 
