@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006 Xavier Claessens <xclaesse@gmail.com>
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,9 +25,7 @@
 
 #include <glib.h>
 
-#include <libtelepathy/tp-chan.h>
-#include <libtelepathy/tp-constants.h>
-#include <libmissioncontrol/mc-account.h>
+#include <telepathy-glib/channel.h>
 
 #include "empathy-contact.h"
 
@@ -60,8 +58,7 @@ typedef struct {
 } EmpathyPendingInfo;
 
 GType               empathy_tp_group_get_type            (void) G_GNUC_CONST;
-EmpathyTpGroup *    empathy_tp_group_new                 (McAccount          *account,
-							  TpChan             *tp_chan);
+EmpathyTpGroup *    empathy_tp_group_new                 (TpChannel          *channel);
 void                empathy_tp_group_close               (EmpathyTpGroup     *group);
 void                empathy_tp_group_add_members         (EmpathyTpGroup     *group,
 							  GList              *contacts,
@@ -80,10 +77,11 @@ GList *             empathy_tp_group_get_local_pendings  (EmpathyTpGroup     *gr
 GList *             empathy_tp_group_get_remote_pendings (EmpathyTpGroup     *group);
 const gchar *       empathy_tp_group_get_name            (EmpathyTpGroup     *group);
 EmpathyContact *    empathy_tp_group_get_self_contact    (EmpathyTpGroup     *group);
-const gchar *       empathy_tp_group_get_object_path     (EmpathyTpGroup     *group);
-TpChan *            empathy_tp_group_get_channel         (EmpathyTpGroup     *group);
 gboolean            empathy_tp_group_is_member           (EmpathyTpGroup     *group,
 							  EmpathyContact     *contact);
+gboolean            empathy_tp_group_is_ready            (EmpathyTpGroup     *group);
+EmpathyPendingInfo *empathy_tp_group_get_invitation      (EmpathyTpGroup     *group,
+							  EmpathyContact    **remote_contact);
 EmpathyPendingInfo *empathy_pending_info_new             (EmpathyContact     *member,
 							  EmpathyContact     *actor,
 							  const gchar        *message);
