@@ -33,8 +33,9 @@
 #include <glib/gi18n.h>
 
 #include <libxml/uri.h>
-#include <libtelepathy/tp-helpers.h>
-#include <libtelepathy/tp-conn.h>
+#include <telepathy-glib/connection.h>
+#include <telepathy-glib/channel.h>
+#include <telepathy-glib/dbus.h>
 
 #include "empathy-debug.h"
 #include "empathy-utils.h"
@@ -59,7 +60,7 @@ empathy_substring (const gchar *str,
  */
 #define USERCHARS "-A-Za-z0-9"
 #define PASSCHARS "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
-#define HOSTCHARS "-A-Za-z0-9"
+#define HOSTCHARS "-A-Za-z0-9_"
 #define PATHCHARS "-A-Za-z0-9_$.+!*(),;:@&=?/~#%"
 #define SCHEME    "(news:|telnet:|nntp:|file:/|https?:|ftps?:|webcal:)"
 #define USER      "[" USERCHARS "]+(:["PASSCHARS "]+)?"
@@ -336,79 +337,15 @@ empathy_mission_control_new (void)
 	return mc;
 }
 
-gchar *
-empathy_inspect_channel (McAccount *account,
-		         TpChan    *tp_chan)
-{
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
-	g_return_val_if_fail (TELEPATHY_IS_CHAN (tp_chan), NULL);
-
-	return empathy_inspect_handle (account,
-				       tp_chan->handle,
-				       tp_chan->handle_type);
-}
-
-gchar *
-empathy_inspect_handle (McAccount *account,
-			guint      handle,
-			guint      handle_type)
-{
-	MissionControl  *mc;
-	TpConn          *tp_conn;
-	GArray          *handles;
-	gchar          **names;
-	gchar           *name;
-	GError          *error = NULL;
-
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
-	g_return_val_if_fail (handle != 0, NULL);
-	g_return_val_if_fail (handle_type != 0, NULL);
-
-	mc = empathy_mission_control_new ();
-	tp_conn = mission_control_get_connection (mc, account, NULL);
-	g_object_unref (mc);
-
-	if (!tp_conn) {
-		return NULL;
-	}
-
-	/* Get the handle's name */
-	handles = g_array_new (FALSE, FALSE, sizeof (guint));
-	g_array_append_val (handles, handle);
-	if (!tp_conn_inspect_handles (DBUS_G_PROXY (tp_conn),
-				      handle_type,
-				      handles,
-				      &names,
-				      &error)) {
-		empathy_debug (DEBUG_DOMAIN, 
-			      "Couldn't get id: %s",
-			      error ? error->message : "No error given");
-
-		g_clear_error (&error);
-		g_array_free (handles, TRUE);
-		g_object_unref (tp_conn);
-		
-		return NULL;
-	}
-
-	g_array_free (handles, TRUE);
-	name = *names;
-	g_free (names);
-	g_object_unref (tp_conn);
-
-	return name;
-}
-
 void
 empathy_call_with_contact (EmpathyContact *contact)
 {
 #ifdef HAVE_VOIP
 	MissionControl        *mc;
 	McAccount             *account;
-	TpConn                *tp_conn;
+	TpConnection          *connection;
 	gchar                 *object_path;
-	const gchar           *bus_name;
-	TpChan                *new_chan;
+	TpChannel             *channel;
 	EmpathyContactFactory *factory;
 	EmpathyTpGroup        *group;
 	EmpathyContact        *self_contact;
@@ -416,71 +353,55 @@ empathy_call_with_contact (EmpathyContact *contact)
 
 	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
-	/* StreamedMedia channels must have handle=0 and handle_type=none.
-	 * To call a contact we have to add him in the group interface of the
-	 * channel. MissionControl will detect the channel creation and 
-	 * dispatch it to the VoIP chandler automatically. */
-
 	mc = empathy_mission_control_new ();
 	account = empathy_contact_get_account (contact);
-	tp_conn = mission_control_get_connection (mc, account, NULL);
-	/* FIXME: Should be async */
-	if (!tp_conn_request_channel (DBUS_G_PROXY (tp_conn),
-				      TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-				      TP_HANDLE_TYPE_NONE,
-				      0,
-				      FALSE,
-				      &object_path,
-				      &error)) {
+	connection = mission_control_get_tpconnection (mc, account, NULL);
+	tp_connection_run_until_ready (connection, FALSE, NULL, NULL);
+	g_object_unref (mc);
+
+	/* We abuse of suppress_handler, TRUE means OUTGOING. The channel
+	 * will be catched in EmpathyFilter */
+	if (!tp_cli_connection_run_request_channel (connection, -1,
+						    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+						    TP_HANDLE_TYPE_NONE,
+						    0,
+						    TRUE,
+						    &object_path,
+						    &error,
+						    NULL)) {
 		empathy_debug (DEBUG_DOMAIN, 
 			      "Couldn't request channel: %s",
 			      error ? error->message : "No error given");
 		g_clear_error (&error);
-		g_object_unref (mc);
-		g_object_unref (tp_conn);
+		g_object_unref (connection);
 		return;
 	}
 
-	bus_name = dbus_g_proxy_get_bus_name (DBUS_G_PROXY (tp_conn));
-	new_chan = tp_chan_new (tp_get_bus (),
-				bus_name,
-				object_path,
-				TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-				TP_HANDLE_TYPE_NONE,
-				0);
+	channel = tp_channel_new (connection,
+				  object_path, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+				  TP_HANDLE_TYPE_NONE, 0, NULL);
+
+	group = empathy_tp_group_new (channel);
+	empathy_run_until_ready (group);
 
-	group = empathy_tp_group_new (account, new_chan);
 	factory = empathy_contact_factory_new ();
 	self_contact = empathy_contact_factory_get_user (factory, account);
+	empathy_contact_run_until_ready (self_contact,
+					 EMPATHY_CONTACT_READY_HANDLE,
+					 NULL);
+
 	empathy_tp_group_add_member (group, contact, "");
 	empathy_tp_group_add_member (group, self_contact, "");	
 
 	g_object_unref (factory);
 	g_object_unref (self_contact);
 	g_object_unref (group);
-	g_object_unref (mc);
-	g_object_unref (tp_conn);
-	g_object_unref (new_chan);
+	g_object_unref (connection);
+	g_object_unref (channel);
 	g_free (object_path);
 #endif
 }
 
-#ifdef HAVE_VOIP
-static void
-got_handle_cb (EmpathyContact        *contact, 
-	       GParamSpec            *property,
-	       EmpathyContactFactory *factory)
-{
-	g_signal_handlers_disconnect_by_func (contact,
-					      got_handle_cb,
-					      factory);
-
-	empathy_call_with_contact (contact);
-	g_object_unref (factory);
-	g_object_unref (contact);
-}
-#endif
-
 void
 empathy_call_with_contact_id (McAccount *account, const gchar *contact_id)
 {
@@ -490,47 +411,55 @@ empathy_call_with_contact_id (McAccount *account, const gchar *contact_id)
 
 	factory = empathy_contact_factory_new ();
 	contact = empathy_contact_factory_get_from_id (factory, account, contact_id);
+	empathy_contact_run_until_ready (contact, EMPATHY_CONTACT_READY_HANDLE, NULL);
 
-	if (empathy_contact_get_handle (contact) != 0) {
-		empathy_call_with_contact (contact);
-		g_object_unref (contact);
-		g_object_unref (factory);
-	} else {
-		g_signal_connect (contact, "notify::handle",
-				  G_CALLBACK (got_handle_cb),
-				  factory);
-	}
+	empathy_call_with_contact (contact);
+
+	g_object_unref (contact);
+	g_object_unref (factory);
 #endif
 }
 
 void
 empathy_chat_with_contact (EmpathyContact  *contact)
 {
-	MissionControl *mc;
+	MissionControl        *mc;
+	McAccount             *account;
+	TpConnection          *connection;
+
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
 	mc = empathy_mission_control_new ();
-	mission_control_request_channel (mc,
-					 empathy_contact_get_account (contact),
-					 TP_IFACE_CHANNEL_TYPE_TEXT,
-					 empathy_contact_get_handle (contact),
-					 TP_HANDLE_TYPE_CONTACT,
-					 NULL, NULL);
+	account = empathy_contact_get_account (contact);
+	connection = mission_control_get_tpconnection (mc, account, NULL);
+	tp_connection_run_until_ready (connection, FALSE, NULL, NULL);
 	g_object_unref (mc);
+
+	/* We abuse of suppress_handler, TRUE means OUTGOING. The channel
+	 * will be catched in EmpathyFilter */
+	tp_cli_connection_call_request_channel (connection, -1,
+						TP_IFACE_CHANNEL_TYPE_TEXT,
+						TP_HANDLE_TYPE_CONTACT,
+						empathy_contact_get_handle (contact),
+						TRUE,
+						NULL, NULL, NULL, NULL);
+	g_object_unref (connection);
 }
 
 void
 empathy_chat_with_contact_id (McAccount *account, const gchar *contact_id)
 {
-	MissionControl *mc;
+	EmpathyContactFactory *factory;
+	EmpathyContact        *contact;
 
-	mc = empathy_mission_control_new ();
-	mission_control_request_channel_with_string_handle (mc,
-							    account,
-							    TP_IFACE_CHANNEL_TYPE_TEXT,
-							    contact_id,
-							    TP_HANDLE_TYPE_CONTACT,
-							    NULL, NULL);
-	g_object_unref (mc);
+	factory = empathy_contact_factory_new ();
+	contact = empathy_contact_factory_get_from_id (factory, account, contact_id);
+	empathy_contact_run_until_ready (contact, EMPATHY_CONTACT_READY_HANDLE, NULL);
+
+	empathy_chat_with_contact (contact);
+
+	g_object_unref (contact);
+	g_object_unref (factory);
 }
 
 const gchar *
@@ -603,3 +532,223 @@ empathy_presence_from_str (const gchar *str)
 	return MC_PRESENCE_AVAILABLE;
 }
 
+gchar *
+empathy_file_lookup (const gchar *filename, const gchar *subdir)
+{
+	gchar *path;
+
+	if (!subdir) {
+		subdir = ".";
+	}
+
+	path = g_build_filename (g_getenv ("EMPATHY_SRCDIR"), subdir, filename, NULL);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+		g_free (path);
+		path = g_build_filename (DATADIR, "empathy", filename, NULL);
+	}
+
+	return path;
+}
+
+typedef struct {
+	EmpathyRunUntilReadyFunc  func;
+	gpointer                  user_data;
+	GObject                  *object;
+	GMainLoop                *loop;
+} RunUntilReadyData;
+
+static void
+run_until_ready_cb (RunUntilReadyData *data)
+{
+	if (!data->func || data->func (data->object, data->user_data)) {
+		empathy_debug (DEBUG_DOMAIN, "Object %p is ready", data->object);
+		g_main_loop_quit (data->loop);
+	}
+}
+
+static gboolean
+object_is_ready (GObject *object,
+		 gpointer user_data)
+{
+	gboolean ready;
+
+	g_object_get (object, "ready", &ready, NULL);
+
+	return ready;
+}
+
+void
+empathy_run_until_ready_full (gpointer                  object,
+			      const gchar              *signal,
+			      EmpathyRunUntilReadyFunc  func,
+			      gpointer                  user_data,
+			      GMainLoop               **loop)
+{
+	RunUntilReadyData  data;
+	gulong             signal_id;
+
+	g_return_if_fail (G_IS_OBJECT (object));
+	g_return_if_fail (signal != NULL);
+
+	if (func && func (object, user_data)) {
+		return;
+	}
+
+	empathy_debug (DEBUG_DOMAIN, "Starting run until ready for object %p",
+		       object);
+
+	data.func = func;
+	data.user_data = user_data;
+	data.object = object;
+	data.loop = g_main_loop_new (NULL, FALSE);
+
+	signal_id = g_signal_connect_swapped (object, signal,
+					      G_CALLBACK (run_until_ready_cb),
+					      &data);
+	if (loop != NULL) {
+		*loop = data.loop;
+	}
+
+	g_main_loop_run (data.loop);
+
+	if (loop != NULL) {
+		*loop = NULL;
+	}
+
+	g_signal_handler_disconnect (object, signal_id);
+	g_main_loop_unref (data.loop);
+}
+
+void
+empathy_run_until_ready (gpointer object)
+{
+	empathy_run_until_ready_full (object, "notify::ready", object_is_ready,
+				      NULL, NULL);
+}
+
+McAccount *
+empathy_channel_get_account (TpChannel *channel)
+{
+	TpConnection   *connection;
+	McAccount      *account;
+	MissionControl *mc;
+
+	g_object_get (channel, "connection", &connection, NULL);
+	mc = empathy_mission_control_new ();
+	account = mission_control_get_account_for_tpconnection (mc, connection, NULL);
+	g_object_unref (connection);
+	g_object_unref (mc);
+
+	return account;
+}
+
+typedef void (*AccountStatusChangedFunc) (MissionControl           *mc,
+					  TpConnectionStatus        status,
+					  McPresence                presence,
+					  TpConnectionStatusReason  reason,
+					  const gchar              *unique_name,
+					  gpointer                  user_data);
+
+typedef struct {
+	AccountStatusChangedFunc handler;
+	gpointer                 user_data;
+	GClosureNotify           free_func;
+	MissionControl          *mc;
+} AccountStatusChangedData;
+
+typedef struct {
+	TpConnectionStatus        status;
+	McPresence                presence;
+	TpConnectionStatusReason  reason;
+	gchar                    *unique_name;
+	AccountStatusChangedData *data;
+} InvocationData;
+
+static void
+account_status_changed_data_free (gpointer ptr,
+				  GClosure *closure)
+{
+	AccountStatusChangedData *data = ptr;
+
+	if (data->free_func) {
+		data->free_func (data->user_data, closure);
+	}
+	g_object_unref (data->mc);
+	g_slice_free (AccountStatusChangedData, data);
+}
+
+static gboolean
+account_status_changed_invoke_callback (gpointer data)
+{
+	InvocationData *invocation_data = data;
+
+	invocation_data->data->handler (invocation_data->data->mc,
+					invocation_data->status,
+					invocation_data->presence,
+					invocation_data->reason,
+					invocation_data->unique_name,
+					invocation_data->data->user_data);
+
+	g_free (invocation_data->unique_name);
+	g_slice_free (InvocationData, invocation_data);
+
+	return FALSE;
+}
+
+static void
+account_status_changed_cb (MissionControl           *mc,
+			   TpConnectionStatus        status,
+			   McPresence                presence,
+			   TpConnectionStatusReason  reason,
+			   const gchar              *unique_name,
+			   AccountStatusChangedData *data)
+{
+	InvocationData *invocation_data;
+
+	invocation_data = g_slice_new (InvocationData);
+	invocation_data->status = status;
+	invocation_data->presence = presence;
+	invocation_data->reason = reason;
+	invocation_data->unique_name = g_strdup (unique_name);
+	invocation_data->data = data;
+
+	g_idle_add_full (G_PRIORITY_HIGH,
+			 account_status_changed_invoke_callback,
+			 invocation_data, NULL);
+}
+
+gpointer
+empathy_connect_to_account_status_changed (MissionControl *mc,
+					   GCallback       handler,
+					   gpointer        user_data,
+					   GClosureNotify  free_func)
+{
+	AccountStatusChangedData *data;
+
+	g_return_val_if_fail (IS_MISSIONCONTROL (mc), NULL);
+	g_return_val_if_fail (handler != NULL, NULL);
+	
+	data = g_slice_new (AccountStatusChangedData);
+	data->handler = (AccountStatusChangedFunc) handler;
+	data->user_data = user_data;
+	data->free_func = free_func;
+	data->mc = g_object_ref (mc);
+
+	dbus_g_proxy_connect_signal (DBUS_G_PROXY (mc), "AccountStatusChanged",
+				     G_CALLBACK (account_status_changed_cb),
+				     data, account_status_changed_data_free);
+
+	return data;
+}
+
+void
+empathy_disconnect_account_status_changed (gpointer token)
+{
+	AccountStatusChangedData *data = token;
+
+	dbus_g_proxy_disconnect_signal (DBUS_G_PROXY (data->mc),
+					"AccountStatusChanged",
+					G_CALLBACK (account_status_changed_cb),
+					data);
+}
+
