@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,30 +23,33 @@
 #include <config.h>
 
 #include <stdlib.h>
+#include <errno.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 
 #include <libebook/e-book.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include <telepathy-glib/util.h>
 #include <libmissioncontrol/mc-account.h>
-#include <libmissioncontrol/mc-account-monitor.h>
 #include <libmissioncontrol/mission-control.h>
 
 #include <libempathy/empathy-idle.h>
-#include <libempathy/empathy-conf.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-debug.h>
 
-#include <libempathy-gtk/empathy-preferences.h>
-#include <libempathy-gtk/empathy-main-window.h>
-#include <libempathy-gtk/empathy-status-icon.h>
+#include <libempathy-gtk/empathy-conf.h>
+
+#include "empathy-main-window.h"
+#include "empathy-status-icon.h"
+#include "bacon-message-connection.h"
 
 #define DEBUG_DOMAIN "EmpathyMain"
 
+static BaconMessageConnection *connection = NULL;
+
 static void
 service_ended_cb (MissionControl *mc,
 		  gpointer        user_data)
@@ -60,33 +63,67 @@ operation_error_cb (MissionControl *mc,
 		    guint           error_code,
 		    gpointer        user_data)
 {
-	empathy_debug (DEBUG_DOMAIN, "Error code %d during operation %d",
-		      error_code,
-		      operation_id);
-}
+	const gchar *message;
 
-static void
-start_mission_control (EmpathyIdle *idle)
-{
-	McPresence presence;
-
-	presence = empathy_idle_get_state (idle);
-
-	if (presence > MC_PRESENCE_OFFLINE) {
-		/* MC is already running and online, nothing to do */
-		return;
+	switch (error_code) {
+	case MC_DISCONNECTED_ERROR:
+		message = _("Disconnected");
+		break;
+	case MC_INVALID_HANDLE_ERROR:
+		message = _("Invalid handle");
+		break;
+	case MC_NO_MATCHING_CONNECTION_ERROR:
+		message = _("No matching connection");
+		break;
+	case MC_INVALID_ACCOUNT_ERROR:
+		message = _("Invalid account");
+		break;
+	case MC_PRESENCE_FAILURE_ERROR:
+		message = _("Presence failure");
+		break;
+	case MC_NO_ACCOUNTS_ERROR:
+		message = _("No accounts");
+		break;
+	case MC_NETWORK_ERROR:
+		message = _("Network error");
+		break;
+	case MC_CONTACT_DOES_NOT_SUPPORT_VOICE_ERROR:
+		message = _("Contact does not support voice");
+		break;
+	case MC_LOWMEM_ERROR:
+		message = _("Lowmem");
+		break;
+	case MC_CHANNEL_REQUEST_GENERIC_ERROR:
+		message = _("Channel request generic error");
+		break;
+	case MC_CHANNEL_BANNED_ERROR:
+		message = _("Channel banned");
+		break;
+	case MC_CHANNEL_FULL_ERROR:
+		message = _("Channel full");
+		break;
+	case MC_CHANNEL_INVITE_ONLY_ERROR:
+		message = _("Channel invite only");
+		break;
+	default:
+		message = _("Unknown error code");
 	}
 
-	empathy_idle_set_state (idle, MC_PRESENCE_AVAILABLE);
+	empathy_debug (DEBUG_DOMAIN, "Error during operation %d: %s",
+		       operation_id, message);
 }
 
 static void
-account_enabled_cb (McAccountMonitor *monitor,
-		    gchar            *unique_name,
-		    EmpathyIdle      *idle)
+use_nm_notify_cb (EmpathyConf *conf,
+		  const gchar *key,
+		  gpointer     user_data)
 {
-	empathy_debug (DEBUG_DOMAIN, "Account enabled: %s", unique_name);
-	start_mission_control (idle);
+	EmpathyIdle *idle = user_data;
+	gboolean     use_nm;
+
+	if (empathy_conf_get_bool (conf, key, &use_nm)) {
+		empathy_idle_set_use_nm (idle, use_nm);
+	}
 }
 
 static void
@@ -94,24 +131,22 @@ create_salut_account (void)
 {
 	McProfile  *profile;
 	McProtocol *protocol;
-	gboolean    salut_created;
+	gboolean    salut_created = FALSE;
 	McAccount  *account;
 	GList      *accounts;
 	EBook      *book;
 	EContact   *contact;
 	gchar      *nickname = NULL;
-	gchar      *published_name = NULL;
 	gchar      *first_name = NULL;
 	gchar      *last_name = NULL;
 	gchar      *email = NULL;
 	gchar      *jid = NULL;
+	GError     *error = NULL;
 
 	/* Check if we already created a salut account */
-	if (!empathy_conf_get_bool (empathy_conf_get(),
-				    EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
-				    &salut_created)) {
-		return;
-	}
+	empathy_conf_get_bool (empathy_conf_get(),
+			       EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
+			       &salut_created);
 	if (salut_created) {
 		return;
 	}
@@ -129,8 +164,10 @@ create_salut_account (void)
 	g_object_unref (protocol);
 
 	/* Get self EContact from EDS */
-	if (!e_book_get_self (&contact, &book, NULL)) {
-		empathy_debug (DEBUG_DOMAIN, "Failed to get self econtact");
+	if (!e_book_get_self (&contact, &book, &error)) {
+		empathy_debug (DEBUG_DOMAIN, "Failed to get self econtact: %s",
+			       error ? error->message : "No error given");
+		g_clear_error (&error);
 		g_object_unref (profile);
 		return;
 	}
@@ -152,39 +189,31 @@ create_salut_account (void)
 	mc_account_set_display_name (account, _("People nearby"));
 	
 	nickname = e_contact_get (contact, E_CONTACT_NICKNAME);
-	published_name = e_contact_get (contact, E_CONTACT_FULL_NAME);
 	first_name = e_contact_get (contact, E_CONTACT_GIVEN_NAME);
 	last_name = e_contact_get (contact, E_CONTACT_FAMILY_NAME);
 	email = e_contact_get (contact, E_CONTACT_EMAIL_1);
 	jid = e_contact_get (contact, E_CONTACT_IM_JABBER_HOME_1);
 	
-	if (G_STR_EMPTY (nickname) || !tp_strdiff (nickname, "nickname")) {
+	if (!tp_strdiff (nickname, "nickname")) {
 		g_free (nickname);
-		nickname = g_strdup (g_get_user_name ());
-	}
-	if (G_STR_EMPTY (published_name)) {
-		g_free (published_name);
-		published_name = g_strdup (g_get_real_name ());
+		nickname = NULL;
 	}
 
 	empathy_debug (DEBUG_DOMAIN, "Salut account created:\n"
 				     "  nickname=%s\n"
-				     "  published-name=%s\n"
 				     "  first-name=%s\n"
 				     "  last-name=%s\n"
 				     "  email=%s\n"
 				     "  jid=%s\n",
-		       nickname, published_name, first_name, last_name, email, jid);
+		       nickname, first_name, last_name, email, jid);
 
 	mc_account_set_param_string (account, "nickname", nickname ? nickname : "");
-	mc_account_set_param_string (account, "published-name", published_name ? published_name : "");
 	mc_account_set_param_string (account, "first-name", first_name ? first_name : "");
 	mc_account_set_param_string (account, "last-name", last_name ? last_name : "");
 	mc_account_set_param_string (account, "email", email ? email : "");
 	mc_account_set_param_string (account, "jid", jid ? jid : "");
 
 	g_free (nickname);
-	g_free (published_name);
 	g_free (first_name);
 	g_free (last_name);
 	g_free (email);
@@ -195,15 +224,93 @@ create_salut_account (void)
 	g_object_unref (book);
 }
 
+/* The code that handles single-instance and startup notification is
+ * copied from gedit.
+ *
+ * Copyright (C) 2005 - Paolo Maggi 
+ */
+static void
+on_bacon_message_received (const char *message,
+			   gpointer    data)
+{
+	GtkWidget *window = data;
+	guint32    startup_timestamp;
+
+	g_return_if_fail (message != NULL);
+
+	empathy_debug (DEBUG_DOMAIN,
+		       "Other instance launched, presenting the main window "
+		       "(message is '%s')", message);
+
+	startup_timestamp = atoi (message);
+
+	/* Set the proper interaction time on the window.
+	 * Fall back to roundtripping to the X server when we
+	 * don't have the timestamp, e.g. when launched from
+	 * terminal. We also need to make sure that the window
+	 * has been realized otherwise it will not work. lame. */
+	if (startup_timestamp == 0) {
+		/* Work if launched from the terminal */
+		empathy_debug (DEBUG_DOMAIN, "Using X server timestamp as a fallback");
+
+		if (!GTK_WIDGET_REALIZED (window)) {
+			gtk_widget_realize (GTK_WIDGET (window));
+		}
+
+		startup_timestamp = gdk_x11_get_server_time (window->window);
+	}
+
+	gtk_window_present_with_time (GTK_WINDOW (window), startup_timestamp);
+}
+
+static guint32
+get_startup_timestamp ()
+{
+	const gchar *startup_id_env;
+	gchar       *startup_id = NULL;
+	gchar       *time_str;
+	gchar       *end;
+	gulong       retval = 0;
+
+	/* we don't unset the env, since startup-notification
+	 * may still need it */
+	startup_id_env = g_getenv ("DESKTOP_STARTUP_ID");
+	if (startup_id_env == NULL) {
+		goto out;
+	}
+
+	startup_id = g_strdup (startup_id_env);
+
+	time_str = g_strrstr (startup_id, "_TIME");
+	if (time_str == NULL) {
+		goto out;
+	}
+
+	errno = 0;
+
+	/* Skip past the "_TIME" part */
+	time_str += 5;
+
+	retval = strtoul (time_str, &end, 0);
+	if (end == time_str || errno != 0)
+		retval = 0;
+
+ out:
+	g_free (startup_id);
+
+	return (retval > 0) ? retval : 0;
+}
+
 int
 main (int argc, char *argv[])
 {
+	guint32            startup_timestamp;
 	EmpathyStatusIcon *icon;
 	GtkWidget         *window;
 	MissionControl    *mc;
-	McAccountMonitor  *monitor;
 	EmpathyIdle       *idle;
-	gboolean           no_connect = FALSE;
+	gboolean           autoconnect = TRUE;
+	gboolean           no_connect = FALSE; 
 	GError            *error = NULL;
 	GOptionEntry       options[] = {
 		{ "no-connect", 'n',
@@ -213,12 +320,12 @@ main (int argc, char *argv[])
 		{ NULL }
 	};
 
-	empathy_debug_set_log_file_from_env ();
-
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	startup_timestamp = get_startup_timestamp ();
+
 	if (!gtk_init_with_args (&argc, &argv,
 				 _("- Empathy Instant Messenger"),
 				 options, GETTEXT_PACKAGE, &error)) {
@@ -226,20 +333,41 @@ main (int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
+	empathy_debug_set_log_file_from_env ();
+
 	g_set_application_name (PACKAGE_NAME);
 
 	gtk_window_set_default_icon_name ("empathy");
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 					   PKGDATADIR G_DIR_SEPARATOR_S "icons");
-	gnome_vfs_init ();
+
+        /* Setting up the bacon connection */
+	connection = bacon_message_connection_new ("empathy");
+	if (connection != NULL) {
+		if (!bacon_message_connection_get_is_server (connection)) {
+			gchar *message;
+
+			empathy_debug (DEBUG_DOMAIN, "Activating existing instance");
+
+			message = g_strdup_printf ("%" G_GUINT32_FORMAT,
+						   startup_timestamp);
+			bacon_message_connection_send (connection, message);
+
+			/* We never popup a window, so tell startup-notification
+			 * that we are done. */
+			gdk_notify_startup_complete ();
+
+			g_free (message);
+			bacon_message_connection_free (connection);
+
+			return EXIT_SUCCESS;
+		}
+	} else {
+		g_warning ("Cannot create the 'empathy' bacon connection.");
+	}
 
 	/* Setting up MC */
-	monitor = mc_account_monitor_new ();
 	mc = empathy_mission_control_new ();
-	idle = empathy_idle_new ();
-	g_signal_connect (monitor, "account-enabled",
-			  G_CALLBACK (account_enabled_cb),
-			  idle);
 	g_signal_connect (mc, "ServiceEnded",
 			  G_CALLBACK (service_ended_cb),
 			  NULL);
@@ -247,8 +375,20 @@ main (int argc, char *argv[])
 			  G_CALLBACK (operation_error_cb),
 			  NULL);
 
-	if (!no_connect) {
-		start_mission_control (idle);
+	/* Setting up Idle */
+	idle = empathy_idle_new ();
+	empathy_idle_set_auto_away (idle, TRUE);
+	use_nm_notify_cb (empathy_conf_get (), EMPATHY_PREFS_USE_NM, idle);
+	empathy_conf_notify_add (empathy_conf_get (), EMPATHY_PREFS_USE_NM,
+				 use_nm_notify_cb, idle);
+
+	/* Autoconnect */
+	empathy_conf_get_bool (empathy_conf_get(),
+			       EMPATHY_PREFS_AUTOCONNECT,
+			       &autoconnect);
+	if (autoconnect && ! no_connect &&
+	    empathy_idle_get_state (idle) <= MC_PRESENCE_OFFLINE) {
+		empathy_idle_set_state (idle, MC_PRESENCE_AVAILABLE);
 	}
 	
 	create_salut_account ();
@@ -257,11 +397,17 @@ main (int argc, char *argv[])
 	window = empathy_main_window_show ();
 	icon = empathy_status_icon_new (GTK_WINDOW (window));
 
+	if (connection) {
+		/* We se the callback here because we need window */
+		bacon_message_connection_set_callback (connection,
+						       on_bacon_message_received,
+						       window);
+	}
+
 	gtk_main ();
 
 	empathy_idle_set_state (idle, MC_PRESENCE_OFFLINE);
 
-	g_object_unref (monitor);
 	g_object_unref (mc);
 	g_object_unref (idle);
 	g_object_unref (icon);
